@@ -45,6 +45,7 @@ protected:
 	virtual CommMessageOut *message_out();
 	virtual CommMessageIn *message_in();
 	virtual int keep_alive_timeout();
+	virtual int first_timeout();
 	virtual bool init_success();
 	virtual bool finish_once();
 
@@ -100,8 +101,8 @@ private:
 		MySSLWrapper(ProtocolMessage *msg, SSL *ssl) :
 			SSLWrapper(msg, ssl)
 		{ }
-		ProtocolMessage *get_msg() const { return this->msg; }
-		virtual ~MySSLWrapper() { delete this->msg; }
+		ProtocolMessage *get_msg() const { return this->message; }
+		virtual ~MySSLWrapper() { delete this->message; }
 	};
 
 private:
@@ -523,6 +524,11 @@ int ComplexMySQLTask::keep_alive_timeout()
 	return MYSQL_KEEPALIVE_DEFAULT;
 }
 
+int ComplexMySQLTask::first_timeout()
+{
+	return is_user_request_ ? this->watch_timeo : 0;
+}
+
 /*
 +--------------------+---------------------+-----+
 | CHARACTER_SET_NAME | COLLATION_NAME      | ID  |
