@@ -19,15 +19,14 @@
 #ifndef _KAFKA_DATATYPES_H_
 #define _KAFKA_DATATYPES_H_
 
-
 #include <assert.h>
-#include <algorithm>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <utility>
+#include <string.h>
 #include <vector>
 #include <string>
-#include <string.h>
 #include <atomic>
 #include <snappy.h>
 #include <snappy-sinksource.h>
@@ -35,7 +34,6 @@
 #include "rbtree.h"
 #include "kafka_parser.h"
 
-
 namespace protocol
 {
 
@@ -553,30 +551,7 @@ public:
 		return kafka_sasl_set_password(password, this->ptr) == 0;
 	}
 
-	std::string get_sasl_info() const
-	{
-		std::string info;
-		if (strcasecmp(this->ptr->mechanisms, "plain") == 0)
-		{
-			info += this->ptr->mechanisms;
-			info += "|";
-			info += this->ptr->username;
-			info += "|";
-			info += this->ptr->password;
-			info += "|";
-		}
-		else if (strncasecmp(this->ptr->mechanisms, "SCRAM", 5) == 0)
-		{
-			info += this->ptr->mechanisms;
-			info += "|";
-			info += this->ptr->username;
-			info += "|";
-			info += this->ptr->password;
-			info += "|";
-		}
-
-		return info;
-	}
+	std::string get_sasl_info() const;
 
 	bool new_client(kafka_sasl_t *sasl)
 	{
@@ -1284,15 +1259,7 @@ public:
 		return &this->member_vec;
 	}
 
-	static bool cmp(const kafka_member_t *m1, const kafka_member_t *m2)
-	{
-		return strcmp(m1->member_id, m2->member_id) < 0;
-	}
-
-	void sort_by_member()
-	{
-		std::sort(this->member_vec.begin(), this->member_vec.end(), cmp);
-	}
+	void sort_by_member();
 
 private:
 	KafkaMeta *meta;
