@@ -168,6 +168,7 @@ public:
 	void set_send_timeout(int timeout) { this->send_timeo = timeout; }
 	void set_receive_timeout(int timeout) { this->receive_timeo = timeout; }
 	void set_keep_alive(int timeout) { this->keep_alive_timeo = timeout; }
+	void set_watch_timeout(int timeout) { this->watch_timeo = timeout; }
 
 public:
 	/* Do not reply this request. */
@@ -180,6 +181,13 @@ public:
 	/* Push reply data synchronously. */
 	virtual int push(const void *buf, size_t size)
 	{
+		if (this->state != WFT_STATE_TOREPLY &&
+			this->state != WFT_STATE_NOREPLY)
+		{
+			errno = ENOENT;
+			return -1;
+		}
+
 		return this->scheduler->push(buf, size, this);
 	}
 
@@ -199,6 +207,12 @@ public:
 		}
 	}
 
+public:
+	void set_prepare(std::function<void (WFNetworkTask<REQ, RESP> *)> prep)
+	{
+		this->prepare = std::move(prep);
+	}
+
 public:
 	void set_callback(std::function<void (WFNetworkTask<REQ, RESP> *)> cb)
 	{
@@ -209,13 +223,16 @@ protected:
 	virtual int send_timeout() { return this->send_timeo; }
 	virtual int receive_timeout() { return this->receive_timeo; }
 	virtual int keep_alive_timeout() { return this->keep_alive_timeo; }
+	virtual int first_timeout() { return this->watch_timeo; }
 
 protected:
 	int send_timeo;
 	int receive_timeo;
 	int keep_alive_timeo;
+	int watch_timeo;
 	REQ req;
 	RESP resp;
+	std::function<void (WFNetworkTask<REQ, RESP> *)> prepare;
 	std::function<void (WFNetworkTask<REQ, RESP> *)> callback;
 
 protected:
@@ -227,6 +244,7 @@ protected:
 		this->send_timeo = -1;
 		this->receive_timeo = -1;
 		this->keep_alive_timeo = 0;
+		this->watch_timeo = 0;
 		this->target = NULL;
 		this->timeout_reason = TOR_NOT_TIMEOUT;
 		this->user_data = NULL;
@@ -471,7 +489,7 @@ protected:
 class WFMailboxTask : public WFGenericTask
 {
 public:
-	void send(void *msg)
+	virtual void send(void *msg)
 	{
 		*this->mailbox = msg;
 		if (this->flag.exchange(true))
@@ -481,10 +499,7 @@ public:
 		}
 	}
 
-	void **get_mailbox() const
-	{
-		return this->mailbox;
-	}
+	void **get_mailbox() const { return this->mailbox; }
 
 public:
 	void set_callback(std::function<void (WFMailboxTask *)> cb)
@@ -538,6 +553,99 @@ protected:
 	virtual ~WFMailboxTask() { }
 };
 
+class WFSelectorTask : public WFGenericTask
+{
+public:
+	virtual int submit(void *msg)
+	{
+		void *tmp = NULL;
+		int ret = 0;
+
+		if (this->message.compare_exchange_strong(tmp, msg) && msg)
+		{
+			ret = 1;
+			if (this->flag.exchange(true))
+			{
+				this->state = WFT_STATE_SUCCESS;
+				this->subtask_done();
+			}
+		}
+
+		if (--this->nleft == 0)
+		{
+			if (!this->message)
+			{
+				this->state = WFT_STATE_SYS_ERROR;
+				this->error = ENOMSG;
+				this->subtask_done();
+			}
+
+			delete this;
+		}
+
+		return ret;
+	}
+
+	void *get_message() const { return this->message; }
+
+public:
+	void set_callback(std::function<void (WFSelectorTask *)> cb)
+	{
+		this->callback = std::move(cb);
+	}
+
+protected:
+	virtual void dispatch()
+	{
+		if (this->flag.exchange(true))
+		{
+			this->state = WFT_STATE_SUCCESS;
+			this->subtask_done();
+		}
+
+		if (--this->nleft == 0)
+		{
+			if (!this->message)
+			{
+				this->state = WFT_STATE_SYS_ERROR;
+				this->error = ENOMSG;
+				this->subtask_done();
+			}
+
+			delete this;
+		}
+	}
+
+	virtual SubTask *done()
+	{
+		SeriesWork *series = series_of(this);
+
+		if (this->callback)
+			this->callback(this);
+
+		return series->pop();
+	}
+
+protected:
+	std::atomic<void *> message;
+	std::atomic<bool> flag;
+	std::atomic<size_t> nleft;
+	std::function<void (WFSelectorTask *)> callback;
+
+public:
+	WFSelectorTask(size_t candidates,
+				   std::function<void (WFSelectorTask *)>&& cb) :
+		message(NULL),
+		flag(false),
+		nleft(candidates + 1),
+		callback(std::move(cb))
+	{
+	}
+
+protected:
+	virtual ~WFSelectorTask() { }
+};
+
 class WFConditional : public WFGenericTask
 {
 public:
@@ -648,6 +756,7 @@ public:
 		this->create = std::move(create);
 	}
 
+public:
 	void set_callback(std::function<void (WFRepeaterTask *)> cb)
 	{
 		this->callback = std::move(cb);
