@@ -169,9 +169,6 @@ public:
 											off_t offset,
 											fio_callback_t callback);
 
-	/* preadv and pwritev tasks are supported by Linux aio only.
-	 * On macOS or others, you will get an ENOSYS error in callback. */
-
 	static WFFileVIOTask *create_preadv_task(int fd,
 											 const struct iovec *iov,
 											 int iovcnt,
@@ -194,25 +191,25 @@ public:
 
 	/* File tasks with path name. */
 public:
-	static WFFileIOTask *create_pread_task(const std::string& pathname,
+	static WFFileIOTask *create_pread_task(const std::string& path,
 										   void *buf,
 										   size_t count,
 										   off_t offset,
 										   fio_callback_t callback);
 
-	static WFFileIOTask *create_pwrite_task(const std::string& pathname,
+	static WFFileIOTask *create_pwrite_task(const std::string& path,
 											const void *buf,
 											size_t count,
 											off_t offset,
 											fio_callback_t callback);
 
-	static WFFileVIOTask *create_preadv_task(const std::string& pathname,
+	static WFFileVIOTask *create_preadv_task(const std::string& path,
 											 const struct iovec *iov,
 											 int iovcnt,
 											 off_t offset,
 											 fvio_callback_t callback);
 
-	static WFFileVIOTask *create_pwritev_task(const std::string& pathname,
+	static WFFileVIOTask *create_pwritev_task(const std::string& path,
 											  const struct iovec *iov,
 											  int iovcnt,
 											  off_t offset,
@@ -256,8 +253,7 @@ public:
 
 	/* Count by a counter's name. When count_by_name(), it's safe to count
 	 * exceeding target_value. When multiple counters share a same name,
-	 * this operation will be performed on the first created. If no counter
-	 * matches the name, nothing is performed. */
+	 * this operation will be performed on the first created. */
 	static int count_by_name(const std::string& counter_name)
 	{
 		return WFTaskFactory::count_by_name(counter_name, 1);
@@ -298,6 +294,10 @@ public:
 	static int send_by_name(const std::string& mailbox_name, void *msg,
 							size_t max);
 
+	template<typename T>
+	static int send_by_name(const std::string& mailbox_name, T *const msg[],
+							size_t max);
+
 public:
 	static WFSelectorTask *create_selector_task(size_t candidates,
 												selector_callback_t callback)
@@ -330,6 +330,10 @@ public:
 	static int signal_by_name(const std::string& cond_name, void *msg,
 							  size_t max);
 
+	template<typename T>
+	static int signal_by_name(const std::string& cond_name, T *const msg[],
+							  size_t max);
+
 public:
 	static WFConditional *create_guard(const std::string& resource_name,
 									   SubTask *task);
@@ -340,18 +344,8 @@ public:
 	/* The 'guard' is acquired after started, so call 'release_guard' after
 	   and only after the task is finished, typically in its callback.
 	   The function returns 1 if another is signaled, otherwise returns 0. */
-	static int release_guard(const std::string& resource_name)
-	{
-		return WFTaskFactory::release_guard(resource_name, NULL);
-	}
-
 	static int release_guard(const std::string& resource_name, void *msg);
 
-	static int release_guard_safe(const std::string& resource_name)
-	{
-		return WFTaskFactory::release_guard_safe(resource_name, NULL);
-	}
-
 	static int release_guard_safe(const std::string& resource_name, void *msg);
 
 public:
