@@ -26,6 +26,8 @@ void CommRequest::handle(int state, int error)
 	this->error = error;
 	if (error != ETIMEDOUT)
 		this->timeout_reason = TOR_NOT_TIMEOUT;
+	else if (!this->target)
+		this->timeout_reason = TOR_WAIT_TIMEOUT;
 	else if (!this->get_message_out())
 		this->timeout_reason = TOR_CONNECT_TIMEOUT;
 	else
@@ -34,19 +36,3 @@ void CommRequest::handle(int state, int error)
 	this->subtask_done();
 }
 
-void CommRequest::dispatch()
-{
-	if (this->scheduler->request(this, this->object, this->wait_timeout,
-								 &this->target) < 0)
-	{
-		this->state = CS_STATE_ERROR;
-		this->error = errno;
-		if (errno != ETIMEDOUT)
-			this->timeout_reason = TOR_NOT_TIMEOUT;
-		else
-			this->timeout_reason = TOR_WAIT_TIMEOUT;
-
-		this->subtask_done();
-	}
-}
-
