@@ -54,7 +54,7 @@ struct WFGlobalSettings
 	int dns_threads;
 	int poller_threads;
 	int handler_threads;
-	int compute_threads;			///< auto-set by system CPU number if value<=0
+	int compute_threads;			///< auto-set by system CPU number if value<0
 	int fio_max_events;
 	const char *resolv_conf_path;
 	const char *hosts_path;
@@ -124,6 +124,26 @@ public:
 
 	static const char *get_error_string(int state, int error);
 
+	static bool increase_handler_thread()
+	{
+		return WFGlobal::get_scheduler()->increase_handler_thread() == 0;
+	}
+
+	static bool decrease_handler_thread()
+	{
+		return WFGlobal::get_scheduler()->decrease_handler_thread() == 0;
+	}
+
+	static bool increase_compute_thread()
+	{
+		return WFGlobal::get_compute_executor()->increase_thread() == 0;
+	}
+
+	static bool decrease_compute_thread()
+	{
+		return WFGlobal::get_compute_executor()->decrease_thread() == 0;
+	}
+
 	// Internal usage only
 public:
 	static bool is_scheduler_created();
