@@ -13,21 +13,26 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  Authors: Wu Jiaxu (wujiaxu@sogou-inc.com)
+  Authors: Li Yingxin (liyingxin@sogou-inc.com)
+           Wang Zhulei (wangzhulei@sogou-inc.com)
 */
 
 #ifndef _UPSTREAMPOLICIES_H_
 #define _UPSTREAMPOLICIES_H_
 
 #include <utility>
+#include <map>
 #include <vector>
+#include <atomic>
 #include <functional>
 #include "URIParser.h"
 #include "EndpointParams.h"
 #include "WFNameService.h"
 #include "WFServiceGovernance.h"
 
-using upstream_route_t = std::function<unsigned int (const char *, const char *, const char *)>;
+using upstream_route_t = std::function<unsigned int (const char *path,
+													 const char *query,
+													 const char *fragment)>;
 
 class EndpointGroup;
 class UPSGroupPolicy;
@@ -39,7 +44,6 @@ public:
 	short server_type;
 	int group_id;
 	EndpointGroup *group;
-	unsigned int consistent_hash[VIRTUAL_GROUP_SIZE];
 
 	UPSAddrParams(const struct AddressParams *params,
 				  const std::string& address);
@@ -49,8 +53,9 @@ class UPSGroupPolicy : public WFServiceGovernance
 {
 public:
 	UPSGroupPolicy();
-	~UPSGroupPolicy();
+	virtual ~UPSGroupPolicy();
 
+public:
 	virtual bool select(const ParsedURI& uri, WFNSTracing *tracing,
 						EndpointAddress **addr);
 	virtual void add_server(const std::string& address,
@@ -71,11 +76,40 @@ protected:
 	virtual void add_server_locked(EndpointAddress *addr);
 	virtual int remove_server_locked(const std::string& address);
 
-	EndpointAddress *consistent_hash_with_group(unsigned int hash);
-	EndpointAddress *check_and_get(EndpointAddress *addr,
-								   bool flag, WFNSTracing *tracing);
+	EndpointAddress *check_and_get(EndpointAddress *addr, bool addr_failed,
+								   WFNSTracing *tracing);
 
 	bool is_alive(const EndpointAddress *addr) const;
+
+protected:
+	EndpointAddress *consistent_hash_with_group(unsigned int hash,
+												WFNSTracing *tracing);
+	void hash_map_add_addr(EndpointAddress *addr);
+	void hash_map_remove_addr(const std::string& address);
+
+	std::map<unsigned int, EndpointAddress *> addr_hash;
+};
+
+class UPSRoundRobinPolicy : public UPSGroupPolicy
+{
+public:
+	UPSRoundRobinPolicy(bool try_another) :
+		cur_idx(0)
+	{
+		this->try_another = try_another;
+	}
+
+protected:
+	virtual EndpointAddress *first_strategy(const ParsedURI& uri,
+											WFNSTracing *tracing);
+	virtual EndpointAddress *another_strategy(const ParsedURI& uri,
+											  WFNSTracing *tracing);
+
+protected:
+	virtual int remove_server_locked(const std::string& address);
+
+protected:
+	std::atomic<size_t> cur_idx;
 };
 
 class UPSWeightedRandomPolicy : public UPSGroupPolicy
@@ -87,10 +121,12 @@ public:
 		this->available_weight = 0;
 		this->try_another = try_another;
 	}
-	EndpointAddress *first_strategy(const ParsedURI& uri,
-									WFNSTracing *tracing);
-	EndpointAddress *another_strategy(const ParsedURI& uri,
-									  WFNSTracing *tracing);
+
+protected:
+	virtual EndpointAddress *first_strategy(const ParsedURI& uri,
+											WFNSTracing *tracing);
+	virtual EndpointAddress *another_strategy(const ParsedURI& uri,
+											  WFNSTracing *tracing);
 
 protected:
 	virtual void add_server_locked(EndpointAddress *addr);
@@ -112,8 +148,10 @@ public:
 		this->cur_idx = 0;
 		this->try_another = false;
 	};
-	EndpointAddress *first_strategy(const ParsedURI& uri,
-									WFNSTracing *tracing);
+
+protected:
+	virtual EndpointAddress *first_strategy(const ParsedURI& uri,
+											WFNSTracing *tracing);
 
 private:
 	virtual void add_server_locked(EndpointAddress *addr);
@@ -122,7 +160,7 @@ private:
 	void init_virtual_nodes();
 	std::vector<size_t> pre_generated_vec;
 	std::vector<int> current_weight_vec;
-	size_t cur_idx;
+	std::atomic<size_t> cur_idx;
 };
 
 class UPSConsistentHashPolicy : public UPSGroupPolicy
@@ -134,10 +172,12 @@ public:
 	}
 
 protected:
-	EndpointAddress *first_strategy(const ParsedURI& uri,
-									WFNSTracing *tracing);
+	virtual EndpointAddress *first_strategy(const ParsedURI& uri,
+											WFNSTracing *tracing);
 
 private:
+	virtual void add_server_locked(EndpointAddress *addr);
+	virtual int remove_server_locked(const std::string& address);
 	upstream_route_t consistent_hash;
 };
 
@@ -158,6 +198,8 @@ public:
 									  WFNSTracing *tracing);
 
 private:
+	virtual void add_server_locked(EndpointAddress *addr);
+	virtual int remove_server_locked(const std::string& address);
 	upstream_route_t manual_select;
 	upstream_route_t another_select;
 };
