@@ -48,13 +48,14 @@ TEST(facilities_unittest, request)
 	req.set_method(HttpMethodGet);
 	req.set_http_version("HTTP/1.1");
 	req.set_request_uri("/");
-	req.set_header_pair("Host", "www.sogou.com");
-	auto res = WFFacilities::request<protocol::HttpRequest, protocol::HttpResponse>(TT_TCP, "http://www.sogou.com", std::move(req), 0);
+	req.set_header_pair("Host", "github.com");
+	auto res = WFFacilities::request<protocol::HttpRequest, protocol::HttpResponse>(TT_TCP, "http://github.com", std::move(req), 0);
 	//EXPECT_EQ(res.task_state, WFT_STATE_SUCCESS);
 	if (res.task_state == WFT_STATE_SUCCESS)
 	{
 		auto code = atoi(res.resp.get_status_code());
-		EXPECT_TRUE(code == HttpStatusMovedPermanently ||
+		EXPECT_TRUE(code == HttpStatusOK ||
+					code == HttpStatusMovedPermanently ||
 					code == HttpStatusFound ||
 					code == HttpStatusSeeOther ||
 					code == HttpStatusTemporaryRedirect ||
@@ -68,13 +69,18 @@ TEST(facilities_unittest, async_request)
 	req.set_method(HttpMethodGet);
 	req.set_http_version("HTTP/1.1");
 	req.set_request_uri("/");
-	req.set_header_pair("Host", "www.sogou.com");
-	auto res = WFFacilities::request<protocol::HttpRequest, protocol::HttpResponse>(TT_TCP_SSL, "https://www.sogou.com", std::move(req), 0);
+	req.set_header_pair("Host", "github.com");
+	auto res = WFFacilities::request<protocol::HttpRequest, protocol::HttpResponse>(TT_TCP_SSL, "https://github.com", std::move(req), 0);
 	//EXPECT_EQ(res.task_state, WFT_STATE_SUCCESS);
 	if (res.task_state == WFT_STATE_SUCCESS)
 	{
 		auto code = atoi(res.resp.get_status_code());
-		EXPECT_EQ(code, HttpStatusOK);
+		EXPECT_TRUE(code == HttpStatusOK ||
+					code == HttpStatusMovedPermanently ||
+					code == HttpStatusFound ||
+					code == HttpStatusSeeOther ||
+					code == HttpStatusTemporaryRedirect ||
+					code == HttpStatusPermanentRedirect);
 	}
 }
 
